package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.platform.bean.*;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description : 通用内存树查询服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-24 11:16
 * @Modify :
 **/
public interface ICoreMemTreeService {

    /**
     * List 分装成树数据
     * @param list      原始List
     * @param parentId  父节点ID
     * @return 多棵树
     */
    @ApiOperation(value = "分装岗位树",notes = "分装岗位树")
    List<SysPosition> packTreeSysPosition(List<SysPosition> list,Long parentId);

    /**
     * 根据用户岗位关系 修剪树的数据
     * @param list      分装好的树
     * @param refList   需要保留的书关系
     * @return 修剪后的树
     */
    @ApiOperation(value = "分装岗位树",notes = "根据用户岗位关系分装岗位树")
    List<SysPosition> packTreeSysPositionBySysRefUserPosition(List<SysPosition> list, List<SysRefUserPosition> refList);

    /**
     * 根据岗位ID 修剪树的数据
     * @param list  分装好的树
     * @param ids   需要保留的书关系
     * @return  修剪后的树
     */
    @ApiOperation(value = "分装岗位树",notes = "根据制定IDS分装岗位树")
    List<SysPosition> packTreeSysPositionByIds(List<SysPosition> list, List<Long> ids);

    /**
     * 根据岗位对象分装书数据
     * @param list
     * @param id
     * @return
     */
    @ApiOperation(value = "分装岗位树",notes = "根据制定ID分装岗位树")
    SysPosition packTreeSysPositionById(List<SysPosition> list, Long id);

    /**
     * 获取制定树的所有叶子节点ID
     * @param position
     * @return
     */
    @ApiOperation(value = "查询岗位子节点",notes = "查询字节点岗位ID")
    List<Long> findChildSysPosition(SysPosition position);

    /********************************************* Start SysOrganize Tree *********************************************/

    @ApiOperation(value = "分装组织树",notes = "分装组织树")
    List<SysOrganize> packTreeSysOrganize(List<SysOrganize> list,Long parentId);

    @ApiOperation(value = "分装组织树",notes = "根据用户组织关系分装组织树")
    List<SysOrganize> packTreeSysOrganizeBySysRefUserOrganize(List<SysOrganize> list, List<SysRefUserDepartment> refList);

    @ApiOperation(value = "分装组织树",notes = "根据用户组织关系分装组织树")
    List<SysOrganize> packTreeSysOrganizeBySysRefUserInfoOrganize(List<SysOrganize> list, List<SysRefUserInfoDepartment> refList);

    @ApiOperation(value = "分装组织树",notes = "根据制定IDS分装组织树")
    List<SysOrganize> packTreeSysOrganizeByIds(List<SysOrganize> list, List<Long> ids);

    @ApiOperation(value = "分装组织树",notes = "根据制定ID分装组织树")
    SysOrganize packTreeSysOrganizeById(List<SysOrganize> list, Long id);

    @ApiOperation(value = "查询组织子节点",notes = "查询字节点组织ID")
    List<Long> findChildSysOrganize(SysOrganize organize);

    /********************************************* End SysOrganize Tree *********************************************/
}
