package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPositionService;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysOrganize;
import cn.estsh.i3plus.pojo.platform.bean.SysPosition;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserDepartment;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserPosition;
import cn.estsh.i3plus.pojo.platform.repository.SysRefUserPositionRepository;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

/**
 * @Description : 内存树
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-07-04 下午12:52
 * @Modify:
 **/
@RestController
@Api(description = "内存树服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/mem-tree")
public class SysMemTreeController extends CoreBaseController {

    @Autowired
    private ISysPositionService positionService;

    @Autowired
    private ICoreMemTreeService memTreeService;

    @Autowired
    private IPersonnelService personnelService;

    /**
     * 根据父节点ID 获取树列表
     * @return 处理结果
     */
    @GetMapping(value="/position/root")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionTree() {
        try {
            List list = positionService.listSysPosition();
            List treeList = memTreeService.packTreeSysPosition(list, CommonEnumUtil.PARENT.DEFAULT.getValue());

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(treeList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据用户ID 获取用户组织树
     * @param userId
     * @return
     */
    @GetMapping(value="/position/user")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionParentId(Long userId) {
        try {
            List list = positionService.listSysPosition();
            List listTree = memTreeService.packTreeSysPosition(list, CommonEnumUtil.PARENT.DEFAULT.getValue());
            List<SysRefUserPosition> refList = personnelService.findSysRefUserPositionById(getSessionUser().getUserInfo().getId());
            List userTreeList = memTreeService.packTreeSysPositionBySysRefUserPosition(listTree, refList);

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(userTreeList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 分装树
     * @param ids
     * @return
     */
    @GetMapping(value="/position/ids")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionParentId(Long[] ids) {
        try {
            List list = positionService.listSysPosition();
            List listTree = memTreeService.packTreeSysPosition(list, CommonEnumUtil.PARENT.DEFAULT.getValue());
            List dataListTree = memTreeService.packTreeSysPositionByIds(listTree, Arrays.asList(ids));
            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(dataListTree);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 获取指定ID 节点的树 , 自动补充跟节点
     * @param id
     * @return
     */
    @GetMapping(value="/position/id")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionById(Long id) {
        try {
            List list = positionService.listSysPosition();
            SysPosition position = memTreeService.packTreeSysPositionById(list,id);

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(position);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 获取字节点所有的ID
     * @param id
     * @return
     */
    @GetMapping(value="/position/child")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean listSysPositionChild(Long id) {
        try {
            List list = positionService.listSysPosition();
            SysPosition position = memTreeService.packTreeSysPositionById(list,id);

            return ResultBean.success("查询部门树成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(memTreeService.findChildSysPosition(position));
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据用户ID 获取用户组织树
     * @return
     */
    @GetMapping(value="/organize/user")
    @ApiOperation(value="查询岗位树",notes="通过部门主键获取岗位树，-1为根节点")
    public ResultBean findSysOrganizeByUser() {
        try {
            List<SysOrganize> list = personnelService.findSysOrganizeList();
            List listTree = memTreeService.packTreeSysOrganize(list, CommonEnumUtil.PARENT.DEFAULT.getValue());
            List<SysRefUserDepartment> refList = personnelService.findSysRefUserDepartmentByUserId(getSessionUser().getUserInfo().getId());
            List userTreeList = memTreeService.packTreeSysOrganizeBySysRefUserOrganize(listTree, refList);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(userTreeList);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
