package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.*;
import io.swagger.annotations.ApiOperation;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-07-04 下午12:46
 * @Modify:
 **/
@Service
public class CoreMemTreeService implements ICoreMemTreeService {

    @Override
    public List<SysPosition> packTreeSysPosition(List<SysPosition> list, Long parentId) {
        List<SysPosition> result = new ArrayList<>();
        List<SysPosition> copyList = new ArrayList<>(list);

        if(list != null && list.size() > 0){
            list.forEach(position -> {
                if(position.getParentId() != null && position.getParentId().equals(parentId)){
                    result.add(position);
                    copyList.remove(position);
                    position.setChildList(packTreeSysPosition(copyList, position.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysPosition> packTreeSysPositionBySysRefUserPosition(List<SysPosition> list, List<SysRefUserPosition> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref ->{
                ids.add(ref.getPositionId());
            });
        }

        return packTreeSysPositionByIds(list, ids);
    }

    @Override
    public List<SysPosition> packTreeSysPositionByIds(List<SysPosition> list, List<Long> ids) {
        List<SysPosition> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            list.forEach(position -> {
                if(position.getChildList() == null || position.getChildList().size() <= 0){
                    if(ids.contains(position.getId())){
                        result.add(position);
                    }
                }else{
                    List<SysPosition> childList = packTreeSysPositionByIds(position.getChildList(), ids);
                    if(ids.contains(position.getId()) || childList != null && childList.size() > 0){
                        position.setChildList(childList);
                        result.add(position);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysPosition packTreeSysPositionById(List<SysPosition> list, Long id) {
        if(list != null && list.size() > 0){
            for (SysPosition position : list) {
                if(position.getId().equals(id)){
                    position.setChildList(packTreeSysPosition(list, position.getId()));
                    return position;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysPosition(SysPosition position) {
        List<Long> result = new ArrayList<>();

        if(position != null && position.getChildList() != null && position.getChildList().size() > 0){
            for (SysPosition sp : position.getChildList()) {
                result.add(sp.getId());
                result.addAll(findChildSysPosition(sp));
            }
        }

        return result;
    }


    /********************************************* Start SysOrganize Tree *********************************************/
    @Override
    public List<SysOrganize> packTreeSysOrganize(List<SysOrganize> list, Long parentId) {
        List<SysOrganize> result = new ArrayList<>();
        List<SysOrganize> copyList = new ArrayList<>(list);

        if(list != null && list.size() > 0){
            list.forEach(organize -> {
                if(organize.getParentId() != null && organize.getParentId().equals(parentId)){
                    result.add(organize);
                    copyList.remove(organize);
                    organize.setChildList(packTreeSysOrganize(copyList, organize.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeBySysRefUserOrganize(List<SysOrganize> list, List<SysRefUserDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref ->{
                ids.add(ref.getOrganizeId());
            });
        }

        return packTreeSysOrganizeByIds(list, ids);
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeBySysRefUserInfoOrganize(List<SysOrganize> list, List<SysRefUserInfoDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if(refList != null && refList.size() > 0){
            refList.forEach(ref ->{
                ids.add(ref.getOrganizeId());
            });
        }

        return packTreeSysOrganizeByIds(list, ids);
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeByIds(List<SysOrganize> list, List<Long> ids) {
        List<SysOrganize> result = new ArrayList<>();

        if(list != null && list.size() > 0){
            list.forEach(organize -> {
                if(organize.getChildList() == null || organize.getChildList().size() <= 0){
                    if(ids.contains(organize.getId())){
                        result.add(organize);
                    }
                }else{
                    List<SysOrganize> childList = packTreeSysOrganizeByIds(organize.getChildList(), ids);
                    if(ids.contains(organize.getId()) || childList != null && childList.size() > 0){
                        organize.setChildList(childList);
                        result.add(organize);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysOrganize packTreeSysOrganizeById(List<SysOrganize> list, Long id) {
        if(list != null && list.size() > 0){
            for (SysOrganize organize : list) {
                if(organize.getId().equals(id)){
                    organize.setChildList(packTreeSysOrganize(list, organize.getId()));
                    return organize;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysOrganize(SysOrganize organize) {
        List<Long> result = new ArrayList<>();

        if(organize != null && organize.getChildList() != null && organize.getChildList().size() > 0){
            for (SysOrganize so : organize.getChildList()) {
                result.add(so.getId());
                result.addAll(findChildSysOrganize(so));
            }
        }

        return result;
    }
    /********************************************* End SysOrganize Tree *********************************************/
}
